'use strict';

var cc       = require('ceci-core');
var channels = require('./channels');


exports.timeout = function(ms) {
  var ch = channels.chan();
  var t = setTimeout(function() {
    clearTimeout(t);
    channels.close(ch);
  }, ms);
  return ch;
};


exports.ticker = function(ms) {
  var ch = channels.chan();
  var t;
  var step = function() {
    clearTimeout(t);
    t = setTimeout(step, ms);
    cc.go(wrapGenerator.mark(function() {
      return wrapGenerator(function($ctx0) {
        while (1) switch ($ctx0.next) {
        case 0:
          $ctx0.next = 2;
          return channels.push(ch, null);
        case 2:
          if (!!$ctx0.sent) {
            $ctx0.next = 4;
            break;
          }

          clearTimeout(t);
        case 4:
        case "end":
          return $ctx0.stop();
        }
      }, this);
    }));
  };
  t = setTimeout(step, ms);
  return ch;
};


exports.each = function(fn, input) {
  return cc.go(wrapGenerator.mark(function() {
    var val;

    return wrapGenerator(function($ctx1) {
      while (1) switch ($ctx1.next) {
      case 0:
        $ctx1.next = 2;
        return channels.pull(input);
      case 2:
        $ctx1.t0 = val = $ctx1.sent;

        if (!(undefined !== $ctx1.t0)) {
          $ctx1.next = 9;
          break;
        }

        if (!fn) {
          $ctx1.next = 7;
          break;
        }

        $ctx1.next = 7;
        return fn(val);
      case 7:
        $ctx1.next = 0;
        break;
      case 9:
      case "end":
        return $ctx1.stop();
      }
    }, this);
  }));
};


exports.fromGenerator = function(gen, output) {
  var managed = output == null;
  if (managed)
    output = channels.chan();

  cc.go(wrapGenerator.mark(function() {
    var step;

    return wrapGenerator(function($ctx2) {
      while (1) switch ($ctx2.next) {
      case 0:
        if (!true) {
          $ctx2.next = 14;
          break;
        }

        step = gen.next();

        if (!step.done) {
          $ctx2.next = 6;
          break;
        }

        delete $ctx2.thrown;
        $ctx2.next = 14;
        break;
      case 6:
        $ctx2.next = 8;
        return channels.push(output, step.value);
      case 8:
        if (!!$ctx2.sent) {
          $ctx2.next = 12;
          break;
        }

        delete $ctx2.thrown;
        $ctx2.next = 14;
        break;
      case 12:
        $ctx2.next = 0;
        break;
      case 14:
        if (managed)
          channels.close(output);
      case 15:
      case "end":
        return $ctx2.stop();
      }
    }, this);
  }));

  return output;
};


function Lock() {
  this.busy = channels.chan();
  this.release();
};

Lock.prototype = {
  acquire: function() {
    return channels.pull(this.busy);
  },
  release: function() {
    channels.push(this.busy, null);
  }
};


exports.createLock = function() {
  return new Lock();
};


exports.fromStream = function(stream, output, onOutputClosed)
{
  var lock = exports.createLock();

  var managed = output == null;
  if (managed)
    output = channels.chan();

  stream.on('readable', function() {
    cc.go(wrapGenerator.mark(function() {
      var chunk;

      return wrapGenerator(function($ctx3) {
        while (1) switch ($ctx3.next) {
        case 0:
          $ctx3.next = 2;
          return lock.acquire();
        case 2:
          if (!(null !== (chunk = stream.read()))) {
            $ctx3.next = 12;
            break;
          }

          $ctx3.next = 5;
          return channels.push(output, chunk);
        case 5:
          if (!!$ctx3.sent) {
            $ctx3.next = 10;
            break;
          }

          if (onOutputClosed)
            onOutputClosed();

          delete $ctx3.thrown;
          $ctx3.next = 13;
          break;
        case 10:
          $ctx3.next = 2;
          break;
        case 12:
          lock.release();
        case 13:
        case "end":
          return $ctx3.stop();
        }
      }, this);
    }));
  });

  stream.on('end', function() {
    if (managed)
      cc.go(wrapGenerator.mark(function() {
        return wrapGenerator(function($ctx4) {
          while (1) switch ($ctx4.next) {
          case 0:
            $ctx4.next = 2;
            return lock.acquire();
          case 2:
            channels.close(output);
          case 3:
          case "end":
            return $ctx4.stop();
          }
        }, this);
      }));
  });

  stream.on('error', function(err) {
    throw new Error(err);
  });

  return output;
};
